/*
 * PaintGraph.java
 *
 * Created on 19 de marzo de 2005, 12:53
 */
package BOM.paint;

import BOM.TextConst;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;

/**
 * 
 * @author Alejandro Touza
 */
public class PaintText {
    
   protected Point textStartPoint;
    
   protected Font textFont;
   protected Color textActiveColor;
   protected Color textUnactiveColor;
   protected Color textSelectedColor;
   protected Dimension textPaintingDimension;
   /** Separacin entre caracteres del texto. (horizontal, vertical) .*/
   protected Dimension textGap;
   protected Dimension cellSize;
   protected double fontRatioWidth;
   protected double fontRatioHeight;
   
   protected Dimension textRealSize;
   protected Color backgroundColor;  
        
   protected Color backRectangleBorderColor;
   protected Color backRectangleInsideColor;
   protected Color backRectangleShadowColor;
   protected Dimension backRectangleShadowSize;  
    
   // CONSTRUCTORAS
   /** Creates a new instance of PaintGraph */
   public PaintText() {
        
        setDefaultValues();
    }
   
   public Font getTextFont(){
        
        return textFont;
    }
   public void setTextFont(Font f){
        
        textFont = f;
    }
   
   public Color getBackgroundColor(){
        
        return backgroundColor;
    }
   public void setBackgroundColor(Color c){
        
        backgroundColor = c;
    }
   public Color getTextActiveColor(){
        
        return textActiveColor;
    }
   public Color getTextUnactiveColor(){
        
        return textUnactiveColor;
    }
   public Color getTextSelectedColor(){
        
        return textSelectedColor;
    }
   public Point getTextStartPoint(){
        
        return textStartPoint;
    }    
   public Dimension getCellSize(){
        
        return cellSize;
    }
   public void setTextActiveColor(Color c){
        
        textActiveColor = c;
    }
    
   public void setTextStartPoint(Point p){
        
        textStartPoint = p;
    }
    
   public void setTextGapWidth(int w){
        
        textGap.width   = w;
        cellSize.width  = textPaintingDimension.width  + textGap.width;
    }
    
   public void setTextGapHeight(int h){
        
        textGap.height  = h;
        cellSize.height = textPaintingDimension.height + textGap.height;
    }
    
   public void setTextGap(int w, int h){
        
        textGap.width  = w;
        textGap.height = h;
        setCellSize();
    }
    
   public void setTextGap(Dimension d){
        
        textGap.width  = d.width;
        textGap.height = d.height;
        setCellSize();
    }
    
   public Dimension getTextGap(){
        
        return textGap;
    }
    
   public void setDefaultValues(){
        
        textStartPoint        = TextConst.TEXT_START_POINT;
        textFont              = TextConst.TEXT_FONT;
        textActiveColor       = TextConst.TEXT_ACTIVE_COLOR;
        textUnactiveColor     = TextConst.TEXT_UNACTIVE_COLOR;
        textSelectedColor     = TextConst.TEXT_SELECTED_COLOR;
        
        textPaintingDimension = new Dimension(20, 20);
        textGap               = TextConst.TEXT_GAP;
        cellSize              = new Dimension(textPaintingDimension.width  + textGap.width,
                                              textPaintingDimension.height + textGap.height);
        fontRatioWidth             = 1;
        
        backgroundColor       = TextConst.BACKGROUND_COLOR;
         
        backRectangleBorderColor  = TextConst.BACK_RECTANGLE_BORDER_COLOR;
        backRectangleInsideColor  = TextConst.BACK_RECTANGLE_INSIDE_COLOR;
        backRectangleShadowColor  = TextConst.BACK_RECTANGLE_SHADOW_COLOR;
        backRectangleShadowSize   = new Dimension (TextConst.BACK_RECTANGLE_SHADOW_WIDTH, 
                                                  TextConst.BACK_RECTANGLE_SHADOW_HEIGHT);
        
        
    }
   /** Calcula la anchura y altura que ser tenida en cuenta para las filas y columnas. Hay
    *  que tener en cuenta que la anchura de las letras de un texto (maysculas y minsculas) 
    *  es diferente por lo que se usar la mayor de las anchuras para pintar a todas. 
    *  @param g Graphics2D que se usar para pintar el texto, se usar la fuente textFont para
    *           calcular las dimensiones a usar.
    *  @param text Son los diferentes caracteres que se piensan pintar haciendo uso de PaintText,
    *              puede contener caracteres repetidos (aunque la bsqueda ser ms lenta).*/ 
   public void calculateTextPaintingDimension(Graphics2D g, String text){
       
       g.setFont(textFont); //Ponemos la fuente que usaremos
       
       //Calculamos la anchura mxima de entre todas las letras del texto (es necesario que sea
       //como mnimo 1 para evitar divisiones por 0 en caso de que el tamao de fuente actual de
       //g sea muy pequeo.
       textPaintingDimension.width = Math.max(g.getFontMetrics().charWidth(text.charAt(0)), 1);
       for (int i=1; i<text.length(); i++)
       {
            textPaintingDimension.width = Math.max(textPaintingDimension.width, 
                                          g.getFontMetrics().charWidth(text.charAt(i)));
       }
       //Calculamos la altura de la fuente. De nuevo es necesario asegurarse del mnimo de altura
       textPaintingDimension.height = Math.max(g.getFontMetrics().getAscent() - 
                                      g.getFontMetrics().getDescent(), 1);
       
       //Calculamos la anchura y altura que usaremos para pintar los caracteres
       setCellSize();
       
       //Calculamos el ratio puntos / pixels
       fontRatioWidth = textFont.getSize()/textPaintingDimension.width;
       fontRatioHeight = textFont.getSize()/textPaintingDimension.height;
   }
   
   public void setTextPaintingDimension(Dimension d){
       
       textPaintingDimension.width = d.width;
       textPaintingDimension.height = d.height;
       
       setCellSize();
   }
   
   private void setCellSize(){
       
       //Calculamos la anchura y altura que usaremos para pintar los caracteres
       cellSize.width  = textPaintingDimension.width  + textGap.width;
       cellSize.height = textPaintingDimension.height + textGap.height;
   }
   
   public void setTextPaintingDimension(int width, int height){
       
       textPaintingDimension.width = width;
       textPaintingDimension.height = height;
       
       setCellSize();
   }
    
   public Dimension getTextPaintingDimension(){
       
       return textPaintingDimension;
   }
   
   public double getFontRatio(){
       
       return fontRatioWidth;
   }
   
   public double getFontRatioH(){
       
       return fontRatioHeight;
   }
   /** Pinta un string a partir de la celda (row, col) con el color de texto definido
    *  en textActiveColor.
    *  @param g Graphics2D donde se pintar el texto.
    *  @param text String que se pintar.
    *  @param row Fila en donde se pintar el texto.
    *  @param col Columna en donde se pintar el texto. */   
   public void paintActiveText(Graphics2D g, String text, int row, int col){
       
       paintText(g, text, textActiveColor, row, col);
   }
   /** Pinta un string a partir de la celda (row, col) con el color de texto definido
    *  en textUnactiveColor.
    *  @param g Graphics2D donde se pintar el texto.
    *  @param text String que se pintar.
    *  @param row Fila en donde se pintar el texto.
    *  @param col Columna en donde se pintar el texto. */  
   public void paintUnactiveText(Graphics2D g, String text, int row, int col){
       
       paintText(g, text, textUnactiveColor, row, col);
   }
   /** Pinta un string a partir de la celda (row, col) con el color de texto definido
    *  en textSelectedColor.
    *  @param g Graphics2D donde se pintar el texto.
    *  @param text String que se pintar.
    *  @param row Fila en donde se pintar el texto.
    *  @param col Columna en donde se pintar el texto. */  
   public void paintSelectedText(Graphics2D g, String text, int row, int col){
       
        paintText(g, text, textSelectedColor, row, col);
   }
   /** Pinta un string a partir de la celda (row, col) con el color de texto pasado.
    *  @param g Graphics2D donde se pintar el texto.
    *  @param text String que se pintar.
    *  @param c Color del texto que se pintar.
    *  @param row Fila en donde se pintar el texto.
    *  @param col Columna en donde se pintar el texto. */   
   public void paintText(Graphics2D g, String text, Color c, int row, int col){
       
       int x = textStartPoint.x + (col * cellSize.width );
       //Hacemos que el punto y sea el comienzo en coordenadas normales, no el baseline del texto
       int y = textStartPoint.y + cellSize.height + (row * cellSize.height); 
       
       g.setFont(textFont);
       g.setColor(c);
       for(int i=0; i<text.length(); i++)
       {
           //Calculamos la diferencia del tamao del carcter actual con el mximo 
           //y lo dividimos por 2 para centrar el carcter
           int aux = (cellSize.width+1 - g.getFontMetrics().charWidth(text.charAt(i)))/2;
           g.drawString(text.substring(i, i+1), x + aux, y);
           x+=cellSize.width;
       }
   }
   /** Dibuja una tabla con rows filas y cols columnas a partir de la celda (0, 0). 
    *  @param g Graphics2D donde se dibujar la tabla.
    *  @param rows Nmero de filas que se dibujarn.
    *  @param cols Nmero de columnas que se dibujarn. */
   public void drawTable(Graphics2D g, int rows, int cols){
       
       drawTable(g, 0, 0, rows, cols);
   }
   /**
    * Dibuja una tabla con rows filas y cols columnas a partir de la celda (rowStart, colStart).
    * @param g Graphics2D donde se dibujar la tabla.
    * @param rowStart Fila donde comienza el dibujado de la tabla.
    * @param colStart Columna donde comienza el dibujado de la tabla.
    * @param rows Nmero de filas que se dibujarn.
    * @param cols Nmero de columnas que se dibujarn.
    */
   public void drawTable(Graphics2D g, int rowStart, int colStart, int rows, int cols){
       
       int x = textStartPoint.x + colStart*cellSize.width;
       int y = textStartPoint.y + rowStart*cellSize.height + textGap.height/2;
       
       g.setColor(Color.black);
       //Dibujado de las filas
       for(int i=0; i<=rows; i++)
           g.drawLine(x, y + i*cellSize.height, x + cols*cellSize.width, y + i*cellSize.height);
       //Dibujado de las columnas
       for(int i=0; i<=cols; i++)
           g.drawLine(x + i*cellSize.width, y, x + i*cellSize.width, y + rows*cellSize.height);
   }
   /**
    * Dibuja una tabla con rows filas y cols columnas a partir de la celda (rowStart, colStart) 
    *  con el color pasado.
    * @param g Graphics2D donde se dibujar la tabla.
    * @param c Color de fondo de las celdas.
    * @param rowStart Fila donde comienza el dibujado de la tabla.
    * @param colStart Columna donde comienza el dibujado de la tabla.
    * @param rows Nmero de filas que se dibujarn.
    * @param cols Nmero de columnas que se dibujarn. */
   public void drawTableWithInsideColor(Graphics2D g, Color c, int rowStart, int colStart, int rows, int cols){
       
       int x = textStartPoint.x + colStart*cellSize.width;
       int y = textStartPoint.y + rowStart*cellSize.height + textGap.height/2;
       
       fillRect(g, c, rowStart, colStart, rowStart + rows - 1, colStart + cols - 1 ); 
       g.setColor(Color.black);
       //Dibujado de las filas
       for(int i=0; i<=rows; i++)
           g.drawLine(x, y + i*cellSize.height, x + cols*cellSize.width, y + i*cellSize.height);
       //Dibujado de las columnas
       for(int i=0; i<=cols; i++)
           g.drawLine(x + i*cellSize.width, y, x + i*cellSize.width, y + rows*cellSize.height);
   }
   /** Dibuja el fondo para el texto con el borde con Color backRectangleBorderColor, el interior
    *  con backRentangleInsideColor y la sombra con backRentangleShadowColor. Para la sombra se
    *  usan las variables backRentangleShadowWidth y backRentangleShadowHeight.
    *  @param g Graphics2D donde se pintar el rectngulo.
    *  @param x Coordenada x del comienzo del rectngulo.
    *  @param y Coordenada y del comienzo del rectngulo.
    *  @param size Dimensiones del rectngulo. 
    *  @param shadow Indica si le dibujamos sombra o no. */
   public void drawBackgroundRect(Graphics2D g, int x, int y, Dimension size, boolean shadow){
       
        //Rectangle
        g.setStroke(new BasicStroke(3f));
        
        if(shadow)
        {
            g.setColor(backRectangleShadowColor);
            //Sombra del rectngulo
            g.fillRoundRect(x + backRectangleShadowSize.width, y + backRectangleShadowSize.height, 
                            size.width, size.height, 20, 20);
        }
        g.setColor(backRectangleInsideColor);
        //Interior del rectngulo
        g.fillRoundRect(x, y, size.width, size.height, 20, 20);
        g.setColor(backRectangleBorderColor);
        g.drawRoundRect(x, y, size.width, size.height, 20, 20); 
   }
   /** Dibuja un rectngulo negro desde la celda (rowStart, colStart) hasta la celda (rowEnd, colEnd). 
    *  @param g Graphics2D donde se dibujar el rectngulo.
    *  @param rowStart Fila donde comienza el rectngulo.
    *  @param colStart Columna donde comienza el rectngulo.
    *  @param rowEnd Fila donde termina el rectngulo.
    *  @param colEnd Columna donde termina el rectngulo. */
   public void drawRect(Graphics2D g, int rowStart, int colStart, int rowEnd, int colEnd){
       
       drawRect(g, Color.black, rowStart, colStart, rowEnd, colEnd);
   }
   /** Dibuja un rectngulo desde la celda (rowStart, colStart) hasta la celda (rowEnd, colEnd). 
    *  @param g Graphics2D donde se dibujar el rectngulo.
    *  @param c Color del rectngulo.
    *  @param rowStart Fila donde comienza el rectngulo.
    *  @param colStart Columna donde comienza el rectngulo.
    *  @param rowEnd Fila donde termina el rectngulo.
    *  @param colEnd Columna donde termina el rectngulo. */
   public void drawRect(Graphics2D g, Color c, int rowStart, int colStart, int rowEnd, int colEnd){
       
       int x = textStartPoint.x;
       int y = textStartPoint.y + textGap.height/2;
       
       g.setColor(c);
       
       g.drawRect(x + colStart*cellSize.width,  
                  y + rowStart*cellSize.height, 
                  (colEnd - colStart + 1)*cellSize.width, 
                  (rowEnd - rowStart + 1)*cellSize.height);
       
   }
   /** Rellena un rectngulo desde la celda (rowStart, colStart) hasta la celda (rowEnd, colEnd). 
    *  @param g Graphics2D donde se dibujar el rectngulo.
    *  @param c Color del rectngulo.
    *  @param rowStart Fila donde comienza el rectngulo.
    *  @param colStart Columna donde comienza el rectngulo.
    *  @param rowEnd Fila donde termina el rectngulo.
    *  @param colEnd Columna donde termina el rectngulo. */
   public void fillRect(Graphics2D g, Color c, int rowStart, int colStart, int rowEnd, int colEnd){
       
       int x = textStartPoint.x;
       int y = textStartPoint.y + textGap.height/2;
       
       g.setColor(c);
       
       g.fillRect(x + colStart*cellSize.width,  
                  y + rowStart*cellSize.height, 
                  (colEnd - colStart + 1)*cellSize.width, 
                  (rowEnd - rowStart + 1)*cellSize.height);
   }
   /** Dibuja un rectngulo negro con bordes redondeados desde la celda (rowStart, colStart) 
    *  hasta la celda (rowEnd, colEnd). 
    *  @param g Graphics2D donde se dibujar el rectngulo.
    *  @param rowStart Fila donde comienza el rectngulo.
    *  @param colStart Columna donde comienza el rectngulo.
    *  @param rowEnd Fila donde termina el rectngulo.
    *  @param colEnd Columna donde termina el rectngulo. */
   public void drawRoundRect(Graphics2D g, int rowStart, int colStart, int rowEnd, int colEnd){
       
       drawRoundRect(g, Color.black, rowStart, colStart, rowEnd, colEnd);
   }
   /** Dibuja un rectngulo con bordes redondeados desde la celda (rowStart, colStart) 
    *  hasta la celda (rowEnd, colEnd). 
    *  @param g Graphics2D donde se dibujar el rectngulo.
    *  @param c Color del rectngulo.
    *  @param rowStart Fila donde comienza el rectngulo.
    *  @param colStart Columna donde comienza el rectngulo.
    *  @param rowEnd Fila donde termina el rectngulo.
    *  @param colEnd Columna donde termina el rectngulo. */
   public void drawRoundRect(Graphics2D g, Color c, int rowStart, int colStart, int rowEnd, int colEnd){
       
       int x = textStartPoint.x;
       int y = textStartPoint.y + textGap.height/2;
       
       g.setColor(c);
       
       g.drawRoundRect(x + colStart*cellSize.width,  
                  y + rowStart*cellSize.height, 
                  (colEnd - colStart + 1)*cellSize.width, 
                  (rowEnd - rowStart + 1)*cellSize.height, 10, 10);
       
   }
}

