/*
 * PaintGraph.java
 *
 * Created on 19 de marzo de 2005, 12:53
 */

package oracle.paint;


import oracle.FactorOracleConst;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import oracle.Transition;
import oracle.Node;


/**
 *
 * @author Alejandro Touza
 */
public class PaintAutomaton {
    
    final static int GRADOS_ARCO = 180;
    //Variables a nivel de Nodo
    protected Dimension nodeSize;
    protected int       nodeSeparation;
    protected int       nodeTotalWidth;
    protected float     nodeStroke;
    
    protected Color nodeInsideColor; //Color del nodo
    protected Color nodeBorderColor; //Color de los bordes del nodo
    protected Color nodeFinalInsideColor;
    protected Color nodeFinalBorderColor;
    protected Color nodeInitialInsideColor;
    protected Color nodeInitialBorderColor;
    protected Color nodeFinalSpecialInsideColor;
    protected Color nodeFinalSpecialBorderColor;
    protected Color nodeSelectedInsideColor;
    protected Color nodeSelectedBorderColor;
    protected Color nodeCurrentInsideColor;
    protected Color nodeCurrentBorderColor;
    
    protected Font  nodeFont;
    protected Color nodeFontColor;
    
    //Variables a nivel de Transicin
    /** Permite que las transiciones describen un ngulo mayor o menor al marcado
     *  por defecto. Se mide en pxels.*/
    protected int   transitionStart;
    protected int   transitionHeight;
    protected float transitionStroke;
    protected float transitionArrowStroke;
    protected Color transitionColor;
    protected Color transitionSelectedColor;
    protected Color transitionCreatedColor;
    protected Font  transitionFont;
    protected Color transitionFontColor;
    protected int   transitionArrowSize;
    
    //Variables a nivel de Grafo
    protected Point automatonStartPoint;
    protected Color backgroundColor;
    
    protected Font  patternFont;
    protected Color patternFontColor;
    protected Color patternFontSelectedColor;
    
    protected Font tagFont;
    protected Color tagColor;
    protected Color tagFontColor;
    
    // CONSTRUCTORAS
    /** Creates a new instance of PaintGraph */
    public PaintAutomaton() {
        
        setDefaultValues();
    }
    
    // CONSULTORAS
    /** Retorna el valor de nodeSize */
    public Dimension getNodeSize(){
        
        return nodeSize;
    }
    public int   getNodeSeparation(){
        
        return nodeSeparation;
    }
    public int   getNodeTotalWidth (){
        
        return nodeTotalWidth;
    }
    public float getNodeStroke (){
        
        return nodeStroke;
    }
    public Color getNodeInsideColor(){
        
        return nodeInsideColor;
    }
    public Color getNodeBorderColor(){
        
        return nodeBorderColor;
    }
    public Color getNodeFinalInsideColor(){
        
        return nodeFinalInsideColor;
    }
    public Color getNodeFinalBorderColor(){
        
        return nodeFinalBorderColor;
    }
    public Color getNodeInitialInsideColor(){
        
        return nodeInitialInsideColor;
    }
    public Color getNodeInitialBorderColor(){
        
        return nodeInitialBorderColor;
    }
    public Color getNodeSelectedInsideColor(){
        
        return nodeSelectedInsideColor;
    }
    public Color getNodeSelectedBorderColor(){
        
        return nodeSelectedBorderColor;
    }
    public Color getNodeCurrentInsideColor(){
        
        return nodeCurrentInsideColor;
    }
    public Color getNodeCurrentBorderColor(){
       
        return nodeCurrentBorderColor;
    }
    public Font  getNodeFont(){
        
        return nodeFont;
    }
    public Color getNodeFontColor(){
        
        return nodeFontColor;
    }
    public int   getTransitionArrowSize(){
        
        return transitionArrowSize;
    }
    public Color getTransitionColor(){
        
        return transitionColor;
    }
    public Color getTransitionSelectedColor(){
        
        return transitionSelectedColor;
    }    
    public Color getTransitionCreatedColor(){
        
        return transitionCreatedColor;
    }
    public int   getTransitionStart(){
        
        return transitionStart;
    }
    public float getTransitionStroke(){
        
        return transitionStroke;
    }
    public float getTransitionArrowStroke(){
        
        return transitionArrowStroke;
    }
    public int   getTransitionHeight(){
        
        return transitionHeight;
    }
    public Font  getTransitionFont(){
        
        return transitionFont;
    }
    public Color getTransitionFontColor(){
        
        return transitionFontColor;
    }
    
    public Point getGraphStartPoint(){
        
        return automatonStartPoint;
    }
    public Color getBackgroundColor(){
        
        return backgroundColor;
    }
    
    public Font  getPatternFont(){
        
        return patternFont;
    }
    public Color getPatternFontColor(){
        
        return patternFontColor;
    }
    public Color getPatternFontSelectedColor(){
        
        return patternFontSelectedColor;
    }
    
    public Font getTagFont(){
        
        return tagFont;
    }
    public Color getTagColor(){
        
        return tagColor;
    }
    public Color getTagFontColor(){
        
        return tagFontColor;
    }
    
    // MODIFICADORAS
    /** Inicializa PaintGraph con los valores por defecto */
    public void setNodeSize(Dimension d){
        
         nodeSize = d;
         nodeTotalWidth = nodeSize.width + nodeSeparation; 
    }
    public void setNodeSeparation(int i){
        
         nodeSeparation = i;
         nodeTotalWidth = nodeSize.width + nodeSeparation; 
    }
    public void setNodeStroke(float f){
        
         nodeStroke = f;
    }
    public void setNodeInsideColor(Color c){
        
         nodeInsideColor = c;
    }
    public void setNodeBorderColor(Color c){
        
         nodeBorderColor = c;
    }
    public void setNodeFinalInsideColor(Color c){
        
         nodeFinalInsideColor = c;
    }
    public void setNodeFinalBorderColor(Color c){
        
         nodeFinalBorderColor = c;
    }
    public void setNodeInitialInsideColor(Color c){
        
         nodeInitialInsideColor = c;
    }
    public void setNodeInitialBorderColor(Color c){
        
         nodeInitialBorderColor = c;
    }
    public void setNodeFinalSpecialInsideColor(Color c){
        
         nodeFinalSpecialInsideColor = c;
    }
    public void setNodeFinalSpecialBorderColor(Color c){
        
         nodeFinalSpecialBorderColor = c;
    }
    public void setNodeSelectedInsideColor(Color c){
        
         nodeSelectedInsideColor = c;
    }
    public void setNodeSelectedBorderColor(Color c){
        
         nodeSelectedBorderColor = c;
    }
    public void setNodeCurrentInsideColor(Color c){
        
         nodeCurrentInsideColor = c;
    }
    public void setNodeCurrentBorderColor(Color c){
        
         nodeCurrentBorderColor = c;
    }
    public void setNodeFont(Font f){
        
         nodeFont = f;
    }
    public void setNodeFontColor(Color c){
        
         nodeFontColor = c;
    }
    
    public void setTransitionColor(Color c){
        
         transitionColor = c;
    }
    public void setTransitionSelectedColor(Color c){
        
         transitionSelectedColor = c;
    }
    public void setTransitionCreatedColor(Color c){
        
         transitionCreatedColor = c;
    }
    public void setTransitionHeight(int i){
        
         transitionHeight = i;
    }
    public void setTransitionStart(int i){
        
         transitionStart = i;
    }
    public void setTransitionStroke(float stroke){
        
        transitionStroke = stroke;
    }
    public void setTransitionArrowStroke(float f){
        
        transitionArrowStroke = f;
    }
    public void setTransitionFont(Font f){
        
         transitionFont = f;
    }
    public void setTransitionFontColor(Color c){
        
         transitionFontColor = c;
    }
    public void setTransitionArrowSize(int i){
        
        transitionArrowSize = i;
    }
    
    public void setAutomatonStartPoint(Point p){
        
         automatonStartPoint = p;
    }
    public void setBackgroundColor(Color c){
        
         backgroundColor = c;
    }
    
    public void setPatternFont(Font f){
        
         patternFont = f;
    }
    public void setPatternFontColor(Color c){
        
         patternFontColor = c;
    }
    public void setPatternFontSelectedColor(Color c){
        
         patternFontSelectedColor = c;
    }
    
    public void setTagFont(Font f){
        
         tagFont = f;
    }
    public void setTagColor(Color c){
        
         tagColor = c;
    }
    public void setTagFontColor(Color c){
        
         tagFontColor = c;
    }
    
    public void setDefaultValues(){
        
        //Variables a nivel de Nodo
        nodeSize       = FactorOracleConst.NODE_SIZE;
        nodeSeparation = FactorOracleConst.NODE_SEPARATION;
        nodeTotalWidth = nodeSize.width + nodeSeparation; 
        nodeStroke     = FactorOracleConst.NODE_STROKE;
        nodeInsideColor             = FactorOracleConst.NODE_INSIDE_COLOR; //del nodo
        nodeBorderColor             = FactorOracleConst.NODE_BORDER_COLOR; //de los bordes del nodo
        nodeFinalInsideColor        = FactorOracleConst.NODE_FINAL_INSIDE_COLOR;
        nodeFinalBorderColor        = FactorOracleConst.NODE_FINAL_BORDER_COLOR;
        nodeInitialInsideColor      = FactorOracleConst.NODE_INITIAL_INSIDE_COLOR;
        nodeInitialBorderColor      = FactorOracleConst.NODE_INITIAL_BORDER_COLOR;
        nodeFinalSpecialInsideColor = FactorOracleConst.NODE_FINAL_SPECIAL_INSIDE_COLOR;
        nodeFinalSpecialBorderColor = FactorOracleConst.NODE_FINAL_SPECIAL_BORDER_COLOR;
        nodeSelectedInsideColor     = FactorOracleConst.NODE_SELECTED_INSIDE_COLOR;
        nodeSelectedBorderColor     = FactorOracleConst.NODE_SELECTED_BORDER_COLOR;
        nodeCurrentInsideColor      = FactorOracleConst.NODE_CURRENT_INSIDE_COLOR;
        nodeCurrentBorderColor      = FactorOracleConst.NODE_CURRENT_BORDER_COLOR;
        
        nodeFont      = FactorOracleConst.NODE_FONT;
        nodeFontColor = FactorOracleConst.NODE_FONT_COLOR;
        
        //Variables a nivel de Transicin
        transitionColor         = FactorOracleConst.TRANSITION_COLOR;
        transitionSelectedColor = FactorOracleConst.TRANSITION_SELECTED_COLOR;
        transitionCreatedColor  = FactorOracleConst.TRANSITION_CREATED_COLOR;
        transitionHeight        = FactorOracleConst.TRANSITION_HEIGHT;
        transitionStart         = FactorOracleConst.TRANSITION_START;
        transitionStroke        = FactorOracleConst.TRANSITION_STROKE;
        transitionArrowStroke   = FactorOracleConst.TRANSITION_ARROW_STROKE;
        transitionArrowSize     = FactorOracleConst.TRANSITION_ARROW_SIZE;
        
        transitionFont      = FactorOracleConst.TRANSITION_FONT;
        transitionFontColor = FactorOracleConst.TRANSITION_FONT_COLOR;
        
        //Variables a nivel de Grafo
        automatonStartPoint = FactorOracleConst.AUTOMATON_START_POINT;
        backgroundColor = FactorOracleConst.AUTOMATON_BACKGROUND_COLOR;
        
        patternFont              = FactorOracleConst.PATTERN_FONT;
        patternFontColor         = FactorOracleConst.PATTERN_FONT_COLOR;
        patternFontSelectedColor = FactorOracleConst.PATTERN_FONT_SELECTED_COLOR;
        
        tagFont                  = FactorOracleConst.TAG_FONT;
        tagFontColor             = FactorOracleConst.TAG_FONT_COLOR;
        tagColor                 = FactorOracleConst.TAG_COLOR;
    }
    /** Reescala el tamao del nodo del grafo segn la Dimension d pasada y 
     *  adems reescala la fuente y dimensiones de las transiciones y nodo
     *  para adecuarlas al nuevo tamao.*/
    public void resize(Dimension d){
        
        nodeSize = d;
        nodeFont = new Font(nodeFont.getName(), nodeFont.getStyle(), d.width/2);
        nodeSeparation = Math.max(Math.min((d.width *7)/8, 100), 25);
        nodeTotalWidth = nodeSize.width + nodeSeparation; 
        transitionHeight = Math.max(Math.min(d.width/2, FactorOracleConst.TRANSITION_HEIGHT_MAXIMUM), 
                                    FactorOracleConst.TRANSITION_HEIGHT_MINIMUM);
        transitionFont = new Font(transitionFont.getName(), 
                                  transitionFont.getStyle(),
                                  Math.max((transitionHeight * 4)/5, 8));
        transitionArrowSize = Math.max(nodeSize.width / 4, 6);
    }
    
    /** REVISAR COLORES INITIAL_FINAL */
    private Color selectNodeColor(Graphics2D g, Node n){
        
        switch (n.getType()){
           
           case FactorOracleConst.NODE_NORMAL:        return nodeInsideColor;
                                     
           case FactorOracleConst.NODE_INITIAL:       return nodeInitialInsideColor;
                                    
           case FactorOracleConst.NODE_FINAL:         return nodeFinalInsideColor;
                                       
           case FactorOracleConst.NODE_INITIAL_FINAL: return nodeFinalInsideColor;
           
           case FactorOracleConst.NODE_FINAL_SPECIAL: return nodeFinalSpecialInsideColor;
                                                                         
       }
        return Color.white;
    }
    /** REVISAR COLORES INITIAL_FINAL */
    private Color selectNodeBorderColor(Graphics2D g, Node n){
        
        switch (n.getType()){
           
           case FactorOracleConst.NODE_NORMAL:        return nodeBorderColor;
                                       
           case FactorOracleConst.NODE_INITIAL:       return nodeInitialBorderColor;
                                       
           case FactorOracleConst.NODE_FINAL:         return nodeFinalBorderColor;
                                       
           case FactorOracleConst.NODE_INITIAL_FINAL: return nodeFinalBorderColor;
           
           case FactorOracleConst.NODE_FINAL_SPECIAL: return nodeFinalSpecialBorderColor;
       }
       return Color.black;
    }
    
    /** Pinta un nodo y su interior en la posicin calculada por el parametro
     *  graphStartPoint y el nombre de Node. */
    public void paintNode(Graphics2D g, Node n){
       
       int x = automatonStartPoint.x + n.getOrder()*(nodeTotalWidth);
       
       paintNode(g, n, x, automatonStartPoint.y, 
               selectNodeBorderColor(g, n), selectNodeColor(g, n), nodeFontColor, nodeStroke);
    }
    /** Pinta el nodo n en la posicin indicada por x e y, tambin pintamos el
     *  nombre del nodo en su interior. */
    public void paintNode(Graphics2D g, Node n, int x, int y){
       
        paintNode(g, n, x, y, selectNodeBorderColor(g, n), selectNodeColor(g, n), nodeFontColor, nodeStroke);
   }
    /** Pinta el nodo n en la posicin indicada por x e y, tambin pintamos el
     *  nombre del nodo en su interior. Los colores utilizados vienen dados por
     *  nOut, nIn y nFont.*/
    public void paintNode(Graphics2D g, Node n, int x, int y, Color nOut, Color nIn, Color nFont, float stroke){
        
       g.setStroke(new BasicStroke(stroke));
       // Interior Nodo
       g.setColor(nIn);
       g.fillOval(x, y, nodeSize.width, nodeSize.height);
       
       // Borde Nodo
       g.setColor(nOut);
       g.drawOval(x, y, nodeSize.width, nodeSize.height);
       g.drawOval(x-1, y-1, nodeSize.width+2, nodeSize.height+2);
       
       // Tratamiento especial (REHACER PARA FLECHA A NODO INICIAL)
       if(n.getType()==FactorOracleConst.NODE_FINAL || n.getType()==FactorOracleConst.NODE_INITIAL_FINAL
          || n.getType()==FactorOracleConst.NODE_FINAL_SPECIAL )
       {
          g.drawOval(x+3, y+3, nodeSize.width-6, nodeSize.height-6);
          //g.drawOval(x+4, y+4, nodeSize.width-8, nodeSize.height-8);
       }
       if(n.getType()==FactorOracleConst.NODE_INITIAL || n.getType()==FactorOracleConst.NODE_INITIAL_FINAL)
       {
           drawArrow(g, 
                     x-20, automatonStartPoint.y + nodeSize.height/2, // (xStart, yStart)
                     x-2,  automatonStartPoint.y + nodeSize.height/2, // (xEnd, yEnd)
                     10);                                         // arrowSize
       }
           
       // Nombre del interior del nodo
       g.setColor(nFont);
       g.setFont(nodeFont);
       // Marcamos los puntos del interior del nodo donde irn los nmeros
       int xFont = fontX(g, x, nodeSize.width, n.getName());
       //int yFont = fontY(y, nodeSize.height, g.getFont().getSize());
       int yFont = fontY(g, y, nodeSize.height);
       g.drawString(n.getName(), xFont, yFont);
   }
    
    /** Pinta el nodo n y el nombre del nodo en su interior con los colores 
     *  indicados por nodeSelectedBorderColor, nodeSelectedInsideColor y nodeFontColor. */
    public void paintRoughNode(Graphics2D g, Node n){
       
       //Calculamos la posicin x en la que ir cada nodo segn su orden
       int x = automatonStartPoint.x + n.getOrder()*(nodeTotalWidth);
       
       if(n.getOrder()%5==0)
       {
            g.setColor(nodeInsideColor);
            g.fillOval(x, automatonStartPoint.y, nodeSize.width, nodeSize.height);
            
            // Nombre del interior del nodo
            g.setColor(nodeFontColor);
            g.setFont(nodeFont);
            // Marcamos los puntos del interior del nodo donde irn los nmeros
            int xFont = fontX(g, x, nodeSize.width, n.getName());
            int yFont = fontY(g, automatonStartPoint.y, nodeSize.height);
            g.drawString(n.getName(), xFont, yFont);
       }      
       // Borde Nodo
       g.setColor(nodeBorderColor);
       g.drawOval(x, automatonStartPoint.y, nodeSize.width, nodeSize.height);
       
    }
    /** Pinta el nodo n y el nombre del nodo en su interior con los colores 
     *  indicados por nodeSelectedBorderColor, nodeSelectedInsideColor y nodeFontColor. */    
    public void paintSelectedNode(Graphics2D g, Node n){
       
        int x = automatonStartPoint.x + n.getOrder()*(nodeTotalWidth);
        paintNode(g, n, x, automatonStartPoint.y, 
                nodeSelectedBorderColor, nodeSelectedInsideColor, nodeFontColor, nodeStroke);
    }
    /** Pinta el nodo n en la posicin indicada por x e y el nombre del nodo 
     *  en su interior con los colores indicados por nodeSelectedBorderColor, 
     *  nodeSelectedInsideColor y nodeFontColor. */
    public void paintSelectedNode(Graphics2D g, Node n, int x, int y){
       
        paintNode(g, n, x, y, nodeSelectedBorderColor, nodeSelectedInsideColor, nodeFontColor, nodeStroke);
   }
    /** Pinta el nodo n y el nombre del nodo en su interior con los colores 
     *  indicados por nodeCurrentBorderColor, nodeCurrentInsideColor y nodeFontColor. */
    public void paintCurrentNode(Graphics2D g, Node n){
       
        int x = automatonStartPoint.x + n.getOrder()*(nodeTotalWidth);
        paintNode(g, n, x, automatonStartPoint.y, 
                nodeCurrentBorderColor, nodeCurrentInsideColor, nodeFontColor, nodeStroke);
   }
    /** Pinta el nodo n en la posicin indicada por x e y el nombre del nodo 
     *  en su interior con los colores indicados por nodeCurrentBorderColor, 
     *  nodeCurrentInsideColor y nodeFontColor. */
    public void paintCurrentNode(Graphics2D g, Node n, int x, int y){
       
        paintNode(g, n, x, y, nodeCurrentBorderColor, nodeCurrentInsideColor, nodeFontColor, nodeStroke);
   }
    
    /** Dibuja una transicin desde el nodo que indica su origen hasta el nodo que indica
     *  su destino.
     *  @param g Graphics2D donde se realizar el dibujado.
     *  @param t La transition que dibujaremos.
     *  @param up Indica, si la transicin es entre dos nodos no consecutivos, si esta 
     *            se dibuja por encima de los nodos o por debajo de estos.
     *            <b>true</b> indica que se dibuje por encima i <b>false</b> por debajo.
     *  @param trs Es el color del que se dibujar la transicin.
     *  @param fnt Es el color de la fuente de la transicin. */
    public void drawTransition(Graphics2D g, Transition t, boolean up, Color trs, Color fnt, float stroke){
      
       int distanciaEntreNodos, tStart;
       //No contemplamos las transiciones reflexivas.
       if ( (distanciaEntreNodos = Math.abs(t.getStart() - t.getEnd())) == 0 ) return; 
       
       if( t.getStart() > t.getEnd()) tStart = t.getEnd(); 
       else tStart = t.getStart(); 
       
       g.setStroke(new BasicStroke(stroke));
       ////////////////////////////////////////////////////////////////////////////////////
       // h:  Altura a la que llegar nuestra transicin (dependiente de la distancia entre
       //     los nodos inicial y final de la misma). El -1 es porque si los nodos son 
       //     consecutivos dibujaremos una flecha entre ellos en lugar de un arco.
       // xA: Coordenada x donde empieza la caja que contiene el arco (o flecha) 
       //     a dibujar (es la posicin donde empieza el nodo + 1/2 de la anchura del nodo).
       // yA: Coordenada y donde empieza la caja que contiene el arco a dibujar 
       //     (tiene en cuenta la distancia en nodos para no cruzar a otros posibles arcos).
       // xW: Anchura de la caja que contiene el arco a dibujar (es la distancia 
       //     horizontal desde un nodo hasta el nodo destino).
       // yH: Altura de la caja que contiene el arco a dibujar (es el doble de la distancia
       //     desde yA hasta el centro del nodo + la mitad de la altura del nodo + altura
       //     extra h en funcin de la distancia entre los nodos de la transicin para evitar
       //     cruces con otras transiciones). 
       int h  = transitionHeight * (distanciaEntreNodos-1);
       int xA = automatonStartPoint.x + tStart*(nodeTotalWidth) + nodeSize.width/2;
     //int yA = graphStartPoint.y - h - transitionStart;
     //int xW = Math.abs(t.getEnd() - t.getStart()) * nodeTotalWidth;
     //int yH = 2* (transitionStart + h + nodeSize.height/2);*/
       //////////////////////////////////////////////////////////////////////////////////////
       
       g.setColor(trs);
       g.setFont(transitionFont);
       // Si los nodos entre los que se va a producir la transicin son consecutivos
       // dibujamos una flecha en linea recta ()---->(). En ese caso solo haremos uso de la xA, 
       // de todas formas por claridad mostramos los valores que tomarn antes de la comparacin.
       if( distanciaEntreNodos == 1)
       {    
            drawArrow(g, 
                      xA + nodeSize.width/2,                      // xStart
                      automatonStartPoint.y + nodeSize.height/ 2,     // yStart
                      xA + nodeSize.width/2 + nodeSeparation - 1, // xEnd
                      automatonStartPoint.y + nodeSize.height/2,      // yEnd
                      transitionArrowSize);
            
            g.setColor(fnt);
            g.drawString(t.getWord(), 
                         fontX(g, xA + nodeSize.width/2, nodeSeparation, t.getWord()),
                         automatonStartPoint.y + nodeSize.height/2 - 2);
       }
       else // Dibujamos una transicin que NO es una recta
       {    
            int yA = automatonStartPoint.y - h - transitionStart;
            int xW = distanciaEntreNodos * nodeTotalWidth;
            int yH = 2* (transitionStart + h + nodeSize.height/2);
            if (up) g.drawArc(xA, yA, xW, yH, 0, GRADOS_ARCO); 
            else 
            {
                g.drawArc(xA, yA, xW, yH, 0, -GRADOS_ARCO); 
                //Tenemos en cuenta que la coordenada 'y' que dibuja el arco est por encima del nodo,
                //si nuestros arcos son inferiores hace falta situarse en el suelo de la caja para
                //pintar la flecha y el texto en la transicin inferior
                yA += yH;  
            }
            //Pintamos la flecha y el texto de la transicin
            drawArrow(g, xA + xW/2-2, yA, xA + xW/2 +2, yA, transitionArrowSize);
            g.setColor(fnt);
            g.drawString(t.getWord(), fontX(g, xA, xW, t.getWord()), yA - 4);
       }
   }

    public void drawRoughTransition(Graphics2D g, Transition t, boolean up){
      
       int distanciaEntreNodos, tStart;
       //No contemplamos las transiciones reflexivas.
       if ( (distanciaEntreNodos = Math.abs(t.getStart() - t.getEnd())) == 0 ) return; 
       
       if( t.getStart() > t.getEnd()) tStart = t.getEnd(); 
       else tStart = t.getStart(); 
       ////////////////////////////////////////////////////////////////////////////////////
       // h:  Altura a la que llegar nuestra transicin (dependiente de la distancia entre
       //     los nodos inicial y final de la misma). El -1 es porque si los nodos son 
       //     consecutivos dibujaremos una flecha entre ellos en lugar de un arco.
       // xA: Coordenada x donde empieza la caja que contiene el arco (o flecha) 
       //     a dibujar (es la posicin donde empieza el nodo + 1/2 de la anchura del nodo).
       // yA: Coordenada y donde empieza la caja que contiene el arco a dibujar 
       //     (tiene en cuenta la distancia en nodos para no cruzar a otros posibles arcos).
       // xW: Anchura de la caja que contiene el arco a dibujar (es la distancia 
       //     horizontal desde un nodo hasta el nodo destino).
       // yH: Altura de la caja que contiene el arco a dibujar (es el doble de la distancia
       //     desde yA hasta el centro del nodo + la mitad de la altura del nodo + altura
       //     extra h en funcin de la distancia entre los nodos de la transicin para evitar
       //     cruces con otras transiciones). 
       int h  = transitionHeight * (distanciaEntreNodos-1);
       int xA = automatonStartPoint.x + tStart*(nodeTotalWidth) + nodeSize.width/2;
     //int yA = graphStartPoint.y - h - transitionStart;
     //int xW = Math.abs(t.getEnd() - t.getStart()) * nodeTotalWidth;
     //int yH = 2* (transitionStart + h + nodeSize.height/2);*/
       //////////////////////////////////////////////////////////////////////////////////////
       
       g.setColor(transitionColor);
       // Si los nodos entre los que se va a producir la transicin son consecutivos
       // dibujamos una flecha en linea recta ()---->(). En ese caso solo haremos uso de la xA, 
       // de todas formas por claridad mostramos los valores que tomarn antes de la comparacin.
       if( distanciaEntreNodos == 1)
       {    
            g.drawLine(xA + nodeSize.width/2,                      // xStart
                       automatonStartPoint.y + nodeSize.height/ 2,     // yStart
                       xA + nodeSize.width/2 + nodeSeparation - 1, // xEnd
                       automatonStartPoint.y + nodeSize.height/2);     // yEnd
       }
       else // Dibujamos una transicin que NO es una recta
       {    
            int yA = automatonStartPoint.y - h - transitionStart;
            int xW = distanciaEntreNodos * nodeTotalWidth;
            int yH = 2* (transitionStart + h + nodeSize.height/2);
            if (up) g.drawArc(xA, yA, xW, yH, 0, GRADOS_ARCO); 
            else g.drawArc(xA, yA, xW, yH, 0, -GRADOS_ARCO); 
       }
   }
    public void drawTransition(Graphics2D g, Transition t, boolean up){
      
       drawTransition(g, t, up, transitionColor, transitionFontColor, transitionStroke);

   }
    public void drawSelectedTransition(Graphics2D g, Transition t, boolean up){
      
       drawTransition(g, t, up, transitionSelectedColor, transitionFontColor, transitionStroke);
   }
    public void drawCreatedTransition(Graphics2D g, Transition t, boolean up){
      
        drawTransition(g, t, up, transitionCreatedColor, transitionFontColor, transitionStroke);
   }
    
    /** Dibuja una flecha desde (xCenter, yCenter) hasta (x, y). La punta de la flecha tiene
      *  un tamao arrowSize. */
    public void drawArrow(Graphics2D g, int xCenter, int yCenter, int x, int y, int arrowSize) {
      
        drawArrow(g, xCenter, yCenter, x, y, arrowSize, transitionArrowStroke);
    }
    /** Dibuja una flecha desde (xCenter, yCenter) hasta (x, y). La punta de la flecha tiene
      *  un tamao arrowSize y Stroke stroke. */
    public void drawArrow(Graphics2D g, int xCenter, int yCenter, int x, int y, int arrowSize, float stroke) {
      
      // Calculamos la direccin hacia la que mirar la flecha
      double aDir = Math.atan2(xCenter - x, yCenter - y);
      
      // Tamao de la flecha / Up arrowTip & Bottom arrowTip
      int i1 = arrowSize;
      int i2 = (int)(arrowSize/1.5);
      
      //////////////////// CABEZA DE LA FLECHA ///////////////////////////////////////////////
      // Creamos el poligono de la flecha (definimos el contorno desde la punta de la flecha)
      Polygon arrowHead = new Polygon();
      arrowHead.addPoint(x                      , y                      );// punta de la flecha
      arrowHead.addPoint(x + xCor(i1, aDir + .5), y + yCor(i1, aDir + .5));
      arrowHead.addPoint(x + xCor(i2, aDir     ), y + yCor(i2, aDir     ));
      arrowHead.addPoint(x + xCor(i1, aDir - .5), y + yCor(i1, aDir - .5));
      arrowHead.addPoint(x                      , y                      );// punta de la flecha
      ////////////////////////////////////////////////////////////////////////////////////////

      g.drawLine(x, y, xCenter, yCenter);
      g.setStroke(new BasicStroke(stroke));      
      g.fillPolygon(arrowHead); 
      g.setColor(Color.black);  //Color del borde de la flecha
      g.drawPolygon(arrowHead); 
   }
    private static int yCor(int len, double dir) {return (int)(len * Math.cos(dir));}
    private static int xCor(int len, double dir) {return (int)(len * Math.sin(dir));}
    /** Determina la coordenada X donde ha de ir situado el String, de tamao
      * wordSize y fuente de tamao fsize, en una linea que comienza en x
      * que sigue su horizontal hasta una anchura w. Antes de llamarlo, g ha de
      * tener en uso la fuente con la que vamos a pintar el texto. */
    private int fontX(Graphics2D g, int x, int w, String str){
       
      return (int)(x + w/2 - (g.getFontMetrics().stringWidth(str))/2);
   }
   /** Determina la coordenada Y donde ha de ir situado el String, de fuente 
      * con tamao fsize, en una linea que comienza en y
      * que sigue su vertical hasta una altura h */
    private int fontY(Graphics2D g, int posY, int nodeHeight){
       
       return (int)(posY + nodeHeight/2 + 
              (g.getFontMetrics().getAscent() - g.getFontMetrics().getDescent())/2);
   } 

   
   
/******************************************* TESTING AREA ***************************************/
   
    /** Dibuja el texto pasado en un Canvas de Dimensin d. Pinta de manera diferente
     *  los caracteres del pattern comprendidos entre las posiciones startSpecial y
     *  endSpecial */
   public void drawGraphTitle(Graphics2D g, Dimension d, String pattern){
       
        g.setFont(patternFont);
        
        int fsize = g.getFont().getSize();
        int x = fontX(g, 0, d.width, pattern);
        int y = fontY(g, 0, 50);
        
        //Dibujamos la caja que contiene al pattern del grafo
        
        int w = g.getFontMetrics().stringWidth(pattern) + 10;
        
        g.setColor(Color.red);      
        g.drawRect(x-12, y-50/2    , w  , 30   );
        g.setColor(Color.black);
        g.fillRect(x-11, y-50/2 +1 , w-1, 30 -1);
        
        //Dibujamos todo el pattern
        g.setColor(patternFontColor);
        g.drawString(pattern, x-7, y);
   }
   
   
   //PARA SER LLAMADO DESDE MYCANVASORACLE
   /** Si se pone a cierto el boleano descartaCeros no pinta las etiquetas del nodo 0. */
   public void drawNodeTag(Graphics2D g, int nodeNumber, String text, boolean descartaCeros){
       
       if(!(descartaCeros && nodeNumber==0)) 
       {
           g.setFont(tagFont);
           g.setStroke(new BasicStroke(1.5f));
           
           int x = automatonStartPoint.x + nodeNumber*(nodeTotalWidth) + 3*this.nodeSize.width/4;
           int y = automatonStartPoint.y + this.nodeSize.height;
           int w = Math.max(8, g.getFontMetrics().stringWidth(text)) + 11;
        
           g.setColor(tagColor);      
           g.fillRect(x , y, w, 15);
           g.setColor(Color.black);
           g.drawRect(x, y, w, 15);
           
           //Dibujamos el pattern
           g.setColor(tagFontColor);
           g.drawString(text, x + w/2 - (g.getFontMetrics().stringWidth(text))/2, fontY(g, y, 17));
       }
   }
}
